/**************************************************************************************************

Main.C -- (C) Niklas Een, Niklas Srensson, 2004

Read a DIMACS file and apply the SAT-solver to it.

**************************************************************************************************/


#include "Solver.h"
#include <ctime>

void printStats(SolverStats& stats, double cpu_time)
{
    printf("starts                : %"I64_fmt"\n", stats.starts);
    printf("conflicts             : %"I64_fmt"   (%.0f /sec)\n", stats.conflicts   , stats.conflicts   /cpu_time);
    printf("decisions             : %"I64_fmt"   (%.0f /sec)\n", stats.decisions   , stats.decisions   /cpu_time);
    printf("propagations          : %"I64_fmt"   (%.0f /sec)\n", stats.propagations, stats.propagations/cpu_time);
    printf("inspects              : %"I64_fmt"   (%.0f /sec)\n", stats.inspects    , stats.inspects    /cpu_time);
    printf("CPU time              : %g s\n", cpu_time);
}


//=================================================================================================


int main(int argc, char** argv)
{
    Solver      S;
    bool        st;
	Constr_reg cr;
	Constr*c;
	c=new Clause();
	cr.registerConstr(c);
	c=new AtMost();
	cr.registerConstr(c);
	c=new XorConstr();
	cr.registerConstr(c);
	c=new MatrixC();
	cr.registerConstr(c);
	S.verbosity = 0;

	for(int i=2;i<argc;++i){
		if(argv[i][0]=='-')
			if(!argv[i][1]){
				printf("Warning, %i. parameter is incorrect, equal to \"-\".n",i);				
			}else{
				switch(argv[i][1]){
					case 'v':
					case 'V':
						++i;
						if(i==argc)printf("Warning, last parametr has no argument, ignored.\n");
						else  S.verbosity = atoi(argv[i]);
						break;
					default:;
				}
			}else{
				printf("Warning, %i. argument is unknown (equal to \"%s\"). Argument ignored.\n",i,argv[i]);
			}
	}

	if (argc == 1){
        //st = cr.parse_DIMACS(stdin, S);
		printf("Program usage: \n\n");
		printf("%s <input> [-v <level>]\n\n",argv[0]);
		printf("<input> - name of input file.\n");
		printf("<level> - number, determines program verbosity. When level=0 (default)\
			   program prints just summary, when level=2 all information is printed.\n");
		exit(0);
	}
    else{
        FILE* in = fopen(argv[1], "rb");
        if (in == NULL)
            fprintf(stderr, "ERROR! Could not open file: %s\n", argv[1]),
            exit(1);
        st = cr.parse_DIMACS(in, S);
        fclose(in);
    }
    if (!st)
        printf("Trivial problem\nUNSATISFIABLE\n"),
        exit(20);
   
    st = S.solve();
    printStats(S.stats, cpuTime());
	printf("MODEL:\n");
	for(int i=0;i<S.model.size();++i)
	  printf("%i, ",S.model[i]);
	printf("\n");
    printf(st ? "SATISFIABLE\n" : "UNSATISFIABLE\n");
	printf("\n");

    return 0;
}
